; Compteur de vitesse   -   JLF    -    .ASM sur pic 16F628 de MICROCHIP
; Nouvelle version V3.0                                    Le 29/05/2021
; Tabulation pour Notepad++
;
; Affiche la vitesse d'un train  l'chelle HO (1/87) en Km/Heure ramene  l'chelle relle.
; Montage : PIC 16F628 + Quartz externe = 4Mhz
; Site    : http://www.la-tour.info/uts/uts_index.html
;
; Deux types de mesures sont possibles, slectionnables par un interrupteur.
; 1 / Mesure la vitesse sur un tronon de 100 cm, 50 cm, 20 cm ou 10 cm entre deux barrires infrarouge ou par fermeture de contact.
;     On a deux barrires infrarouges banalises (RBcapt_1 et RBcapt_2). Le sens de passage du train devant ces capteurs est indiffrent.
;     Quand le train passe devant un capteur, le comptage commence et s'arrte ds que l'on passe devant l'autre capteur. 
;     Pendant le temps du comptage, la led verte "Tronon" clignote rapidement.
;     Une fois pass le second capteur, on affiche la vitesse et la led "Tronon" clignote lentement, le temps de faire passer tous les wagons.
;     Il faut que plus aucun train ne passe pendant x secondes devant les capteurs, pour rarmer le comptage. 
;     Si le train n'est pas pass au bout de x secondes devant l'autre capteur, on annule le comptage en cours et on allume la led rouge "Erreur".
;
; 2 / Mesure la vitesse sur un tour complet de rseau.
;     Dans ce cas, on utilise qu'un seul capteur (RBcapt_1).
;     Pendant le temps du comptage, la led orange "Tour" clignote rapidement.
;     Une fois repass devant le capteur, on affiche la vitesse et la led "Tour" clignote lentement, le temps de faire passer tous les wagons.
;     Il faut que plus aucun train ne passe pendant x secondes devant ce capteur, pour rarmer le comptage. 
;     Si le train n'est pas repass au bout de xx secondes devant le capteur, on annule le comptage en cours et on allume la led rouge "Erreur".
;     Il faut configurer la distance du tour du rseau dans le programme.
;     Voir les explications  la ligne : "Configuration de la longueur d'un tour de circuit".
;
; Le comptage est donc possible que lorsque qu'aucune led ne clignote.
; Il est possible d'installer que 4 afficheurs sur le circuit imprim, pour n'afficher que les units.
;
; Entres du PIC :
;   Un interrupteur pour choisir le type de mesure : Ouvert = Tronon / Ferm ( la masse) = Tour.
;   Entres pour  deux capteurs de passage = Contact ferm ( la masse) momentanment quand le train passe devant le capteur.
;     Il est prfrable d'avoir un petit dlai quand le contact rest ferm (0,1  1 sec).
;     Il ne faut pas de dlai entre le passage du train et la fermeture du contact  la masse, mais un petit dlai  l'ouverture 
;     du contact (Genre anti rebond) vite de passer un multitude de fois dans la routine d'interruption du PIC.
;   Longueur du tronon = Entres RA4 et RA3 relies au +5 volts ou  la masse.
;     RA4= 0 + RA3= 0 =>  10cm.
;     RA4= 0 + RA3=5V =>  20cm.
;     RA4=5V + RA3= 0 =>  50cm.
;     RA4=5V + RA3=5V => 100cm.
;
; Animation des leds :
;   O = Led verte fixe                = Mesure sur un tronon de 100 cm, 50 cm ou 20 cm. Montage prt, en attente de comptage.
;   O = Led verte clignotante rapide  = Passage du train, en phase de comptage sur ce tronon.
;   O = Led verte clignotante lent    = En fin de phase de comptage sur ce tronon, attend la fin du passage du train.
;
;   O = Led orange fixe               = Mesure sur le tour complet du rseau. Montage prt, en attente de comptage.
;   O = Led orange clignotante rapide = Passage du train, en phase de comptage sur un tour.
;   O = Led orange clignotante lent   = En fin de phase de comptage sur un tour, attend la fin du passage du train.
;
;   O = Led rouge clignotante lent    = Erreur de calcul (temps > temps max ou temps < temps mini). 
;                                       L'affichage ou le calcul de la vitesse n'est pas possible, on affiche alors 0.00
;
; Tronon           Temps max     Vitesse mini en cm/s     Vitesse mini en km/h
;   100 cm            67 sec            1,5                     4,7
;    50 cm            34 sec            1,5                     4,7
;    20 cm            34 sec            0,6                     1,9
;    10 cm            17 sec            0,6                     1,9
;
; Tour du rseau                                                                                          
;    10 m             10 mn                                     5
;    20 m             10 mn                                    11
;    30 m             10 mn                                    17
;   100 m             10 mn                                    60
;
; Explications sur la programmation :
; Je remercie les auteurs de la division 48/24 bits et de la conversion binaire->BCD, sans quoi l'criture de ce programme n'aurait pas t possible. 
; Les cours de Bigonoff sont aussi  l'origine de la conversion d'une ide en ces lignes de code.
;
; A partir de la formule utilise dans ce programme, il est possible d'afficher la vitesse  n'importe quelle chelle en adaptant des variables.
; La formule est : Vitesse = Distance / Temps.
;
; Avec la variable "divi_t" = 100, 50 ou 20(Longueur en cm) * 3,6 * 87(chelle) / top pour tmr1(secondes)
;   On affiche directement la valeur relle de la vitesse du modle rduit.
; Ensuite, exemple  pour 100 cm  une chelle de 1/87
;   Vitesse = 100 * 3,6 * 87 * temps de parcours, pour convertir les mtres/seconde en Km/heure
;
; Sur un tronon de 100  10 cm :
;   Prdiviseur=1,2 ou 4 soit de 1s  4 s. Compteurs = Timer1 + cmpt1.
; Sur un tour de rseau de 30 mtres (Exemple), distance personnalisable :
;   Voir  la ligne : "Configuration de la longueur d'un tour de circuit".
;   Prdiviseur=8 soit 8s. Compteurs = Timer1 + cmpt1 + cmpt2 + dcalage de 2 bits  la fin soit une division par 4, ce qui est quivalent  32s.
;
; Le programme n'est pas d'une criture absolue.
;   Il peut arriver parfois, que les variables partages entre la routine d'interruption et la boucle principale entre en concurrence.
;   Le retard inhrent  la programmation, entre le top dpart et le top fin des capteurs, induit une infime (mais prsente) erreur de mesure.
;   Le programme fonctionne trs bien, mais ne peut pas servir d'affichage d'homologation.
;   Au vu de la prcision utilise, on a 1000.00 Km/h +/- 0,04 Km/h et 100.00 Km/h +/- 0,004 Km/h.
;
;*************************************************************************************
	LIST		P=PIC16F628		; Dfinition de processeur
	#include	<p16F628.inc>	; Dfinitions des variables

	__CONFIG   _CP_OFF & _DATA_CP_OFF & _LVP_OFF & _BODEN_OFF & _MCLRE_ON & _PWRTE_ON & _WDT_OFF & _HS_OSC 
	ERRORLEVEL  -302

	CBLOCK			0x020	; Dbut de la zone variables en BANK0
	cmpt1 : 		1		; Compteur de boucles 1
	cmpt2 : 		1		; Compteur de boucles 2
	cmpt2b :		1		; Compteur temporaire pour dcalage
	digit07 :		1		; Digit pour afficher les leds
	clignot :		1		; Compteur pour le clignotement des leds.
	choixdistance :	1		; Choix du tronon = 100, 50, 20, 10 cm = 4, 3, 2, 1 ou Tour = 0.
	etat1 : 		1		; Etat du programme, voir plus bas les "#define".
	etat2 : 		1		; Etat du programme, voir plus bas les "#define".
	cpttronini :	1		; Compteur de temporisation fin de tronon.
	cpttourini :	1		;  // fin de tour du rseau.
	cpt100ms :		1		; Compteur top 100 msec.
	digitmp	:		1		; Variable temporaire.

;*************************************************************************************
; Donnes pour le sous-programme de temporisation
	delai1 :		1		;
	delai2 :		1		;
	
; Donnes pour le sous-programme de conversion binaire dcimale
	count0 :		1		; En entre : LSD, Valeur sur 3 octets, modifis par le sous programme.
	count1 :		1		;   //
	count2 :		1		;   //  MSB
	digit1 :		1		; En sortie : Digit 0000,0x Ne pas modifier l'ordre de dclaration de ces digits !
	digit2 :		1		;  // Digit 0000,x0
	digit3 :		1		;  // Digit 000x,00
	digit4 :		1		;  // Digit 00x0,00
	digit5 :		1		;  // Digit 0x00,00
	digit6 :		1		;  // Digit x000,00
	digit7 :		1		;  // Digit pas affich
	digit8 :		1		;  // Digit pas affich 
	bitcnt :		1		;
	digcnt :		1		;
	
; Donnes pour le sous-programme de division
	dividend :		6		; msb = octet dividend+0 En entre le "Dividende", puis en retour le "Quotient" calcul sur 48 bits = 6 octets.
	divisor :		3		; msb = octet divisor+0  Diviseur sur 24 bits = 3 octets.
	temp :  		3		;
	temp2 :  		1		;
	bitcount :		1		;
	ENDC					; Fin de la zone

	CBLOCK	0x070 			; Dbut de la zone variables commune  toutes les BANK.
; Donnes pour les sauvegardes dans la routine d'interruption
	w_temp :		1		;
	status_temp :	1		;
	ENDC					; Fin de la zone
	
;*************************************************************************************
; Dfinition des variables.

#define	BANK0		BCF STATUS,RP0	; Active le bank 0
#define	BANK1		BSF STATUS,RP0	; Active le bank 1

;                       PORTA,0,1,2	; Sortie valeur du digit 0  8.
#define	RAchdist_1		PORTA,3		; Entre choix de la longueur du tronon fixe 
#define	RAchdist_2		PORTA,4		; Entre RA4-RA3 : 00=10cm, 01=20cm, 10=50cm, 11=100cm.
;                       PORTA,5		; MCLR = Reset.
;                       PORTA,6,7	; Quartz 4 Mhz.

;                       PORTB,0,1,2,3; Sortie slection du digit 0  7.
#define	RBchtrontr		PORTB,4		; Entre interrupteur choix mesure sur Tronon ou Tour.
;						PORTB,5		; Entre plus utilise dans cette version V3.
#define	RBcapt_1		PORTB,6		; Entre capteur 1 
#define	RBcapt_2		PORTB,7		; Entre capteur 2

; Dfinition des bits de la variable 'etat1'
#define	bit_etat_err	 etat1,0	; 1 = Allume la led "Erreur".
#define	bit_etat_clr	 etat1,1	; 1 = Fait clignoter rapidement les leds, en cours de comptage de la vitesse.
#define	bit_etat_cll	 etat1,2	; 1 = Fait clignoter lentement les leds, en attente de la fin du passage du train entier.
#define	bit_tempotronini etat1,3	; 1 = Temporisation en cours.
#define	bit_tempotourini etat1,4	; 1 = Temporisation en cours.

; Dfinition des bits de la variable 'etat2'
#define	bit_capt_1		 etat2,0	; 1 = Passage effectu devant le capteur 1.
#define	bit_capt_2		 etat2,1	; 1 = Passage effectu devant le capteur 2.
#define	bit_erreur	 	 etat2,2	; 1 = Dpassement de capacit des compteurs
#define	bit_calcul		 etat2,3	; 1 = Calcul la valeur sur les digits.
#define	bit_razprg       etat2,4	; 1 = Razprg (changement de mode de comptage)
#define	bit_digitzero    etat2,5	; 1 = Sert  effacer les zros non significatifs


;*************************************************************************************
; Temporisations pour la mesure la vitesse sur un tronon de 100 cm, 50 cm, 20 cm ou 10.
;   Il faut que plus aucun train ne passe pendant x secondes devant les capteurs, pour rarmer le comptage. 
#define	tempotronini .30	; (Laisser le '.' devant le chiffre pour du dcimal!) en dixime de seconde, 30 = 3 secondes. Minimum 2 = 0,2 sec  maximum 255 = 25 sec.
; Temporisations pour la mesure la vitesse sur un tour complet de rseau.
;   Il faut que plus aucun train ne passe pendant x secondes devant ce capteur, pour rarmer le comptage. 
#define	tempotourini .30	; (Laisser le '.' devant le chiffre pour du dcimal!) En dixime de seconde, 30 = 3 secondes. Minimum 2 = 0,2 sec  maximum 255 = 25 sec.

; Configuration des donnes du programme pour calculer la vitesse d'un tour de circuit.
;   Il faut saisir la distance dans les 6 octets "divi_tx".
;   Exemple si un tour de rseau mesure 3000 cm (30 mtres) de long.
;   Utiliser la calculatrice Windows en mode "Programmeur", pour faire la conversion Dcimal vers HexaDcimal.
;   Utiliser cette calculatrice en mode "Standard", et faire l'opration : 3000 * 3,6 * 87 / 0,000032 = 29362500000.
;   Copier le rsultat (Ctrl-C).
;   Passer cette calculatrice en mode "Programmeur" et choisir le mode "DEC".
;   Coller le rsultat (Ctrl-V), on a les valeurs en DEC = 29362500000 et en HEX = 6D62431A0.
;   Choisir le mode "HEX" et Copier/Coller le rsultat dans ce programme.
;   Rpartir le rsultat Hexadcimale dans les variables ci-dessous, LSB en premire ligne "divi_t1".
#define	divi_t1	0xA0	; <- LSB, Attention l'octet de poids faible est ici en premier.
#define	divi_t2	0x31	; Exemple ici, pour un tour de rseau de 3000 (cm) => 29362500000 soit 6D62431A0 Hexadcimale.	
#define	divi_t3	0x24	; Le top pour tmr1 = 32 secondes (Quartz  4Mhz + Prdiviseur=8 + Dcalage 2bits) d'o la division par 0.000032.
#define	divi_t4	0xD6	; Copier le tout en hexa sur 6 octet = 00 06 D6 24 31 A0 (Exemple ici pour 30000 cm).
#define	divi_t5	0x06	;                                      MSB . . . . . LSB  (<-Master bit et Lower bit).
#define	divi_t6	0x00	; <- MSB, Attention l'octet de poids fort est ici en dernier.

; Configuration des donnes du programme pour calculer la vitesse sur un tronon.
;   Ne pas modifier ces valeurs, sauf si l'on veut un tronon de longueur particulire.
#define	divi_m101 0x80	; Tronon de longueur fixe 100 cm, Vitesse = 100(longueur en cm) * 3,6 * 87(chelle) / 4s => Vitesse = 7.830.000.000 / Nb de top de 4S.
#define	divi_m102 0x51	; Dure du top pour Timer1 = 4 seconde (Quartz  4Mhz + prdiviseur = 4)
#define	divi_m103 0xB4
#define	divi_m104 0xD2	; Le tout en hexa sur 6 octets (7830000000 en dcimal pour 1 mtre). Vitesse minimum affichable = 4,7 Km/h
#define	divi_m105 0x01	; = 00 01 D2 B4 51 80 pour 1 mtre. Rsolution de calcul  1000 Km/h = +/- 0,02 Digit.
#define	divi_m106 0x00	; <- MSB (Attention l'octet de poids fort en dernier)

#define	divi_m51 0x80	; Tronon de longueur fixe 50 cm, Vitesse = 50(longueur en cm) * 3,6 * 87(chelle) / 2s => Vitesse = 7.830.000.000 / Nb de top de 2S.
#define	divi_m52 0x51	; Dure du top pour Timer1 = 2 seconde (Quartz  4Mhz + prdiviseur = 2)
#define	divi_m53 0xB4
#define	divi_m54 0xD2	; Le tout en hexa sur 6 octets (7830000000 en dcimal pour 50 cm). Vitesse minimum affichable = 4,7 Km/h
#define	divi_m55 0x01	; = 00 01 D2 B4 51 80 pour 50 cm. Rsolution de calcul  1000 Km/h = +/- 0,02 Digit.
#define	divi_m56 0x00	; <- MSB (Attention l'octet de poids fort en dernier)

#define	divi_m21 0x00	; Tronon de longueur fixe 20 cm, Vitesse = 20(longueur en cm) * 3,6 * 87(chelle) / 1s => Vitesse = 6.264.000.000 / Nb de top de 1S.
#define	divi_m22 0x0E	; Dure du top pour Timer1 = 1 seconde (Quartz  4Mhz + prdiviseur = 1)
#define	divi_m23 0x5D
#define	divi_m24 0x75	; Le tout en hexa sur 6 octets (6264000000 en dcimal pour 20 cm). Vitesse minimum affichable = 1,9 Km/h
#define	divi_m25 0x01	; = 00 01 75 5D 0E 00 pour 20 cm.
#define	divi_m26 0x00	; <- MSB (Attention l'octet de poids fort en dernier)

#define	divi_m11 0x00	; Tronon de longueur fixe 10 cm, Vitesse = 10(longueur en cm) * 3,6 * 87(chelle) / 1s => Vitesse = 3.132.000.000 / Nb de top de 1S.
#define	divi_m12 0x87	;  Dure du top pour Timer1 = 1 seconde (Quartz  4Mhz + prdiviseur = 1)
#define	divi_m13 0xAE
#define	divi_m14 0xBA	; Le tout en hexa sur 6 octets (3132000000 en dcimal pour 10 cm). Vitesse minimum affichable = 1,9 Km/h
#define	divi_m15 0x00	; = 00 00 BA AE 87 00 pour 10 cm. Rsolution  1000 Km/h = +/- 0,04 Digit.
#define	divi_m16 0x00	; <- MSB (Attention l'octet de poids fort en dernier)

;*************************************************************************************
; Dmarrage du programme  la mise sous tension.
; Attention il ne faut pas crire dans RB4  RB7, sinon a fausse l'interruption sur changement RB47 !
		org			 0x2100 	   ; Initialize EEPROM Data
		dt 			 "Compteur de vitesse V3 "	; Textes visibles dans le '.hex'.
		dt			 "Le 19/05/2021 "	
		dt			 "http://www.la-tour.info/uts/uts_index.html"	

		org 		0x000			; Adresse de dpart aprs raz.
		goto		init
		org 		0x004			; Adresse de dpart aprs interruption.
		goto		interrup
;*************************************************************************************
		dt "__COMPTEUR_DE_VITESSE_V3___"	; Textes visibles dans le '.hex'.
		dt "__Le 19/05/2021___"	
		dt "__http://www.la-tour.info/uts/uts_index.html___"	

; Initialisation des variables.
init	BANK0						; Slectionner banque 0
		bcf 		INTCON,GIE		; Interruptions globalement interdites.
		bcf 		INTCON,GIE		; Interruptions globalement interdites (bis).
		clrf		etat1
		clrf		etat2
		clrf		cpttronini
		clrf		cpttourini
		clrf		cpt100ms

; Initialisation des entres/sorties.
		movlw		0x07			; Pour PIC16F628, mettre les comparateurs analogique  l'arrt.
		movwf		CMCON
		bcf 		T1CON,TMR1CS	; Entre Timer1 = Fosc/4 = 1sec.
		bcf 		T1CON,T1OSCEN	; Pas d'ocillateur externe RB6-RB7 pour l'entre Timer1.
		bsf 		T1CON,NOT_T1SYNC ; Synchro clock pour Timer1
		clrf		CCP1CON			; Pas de module CCP
		BANK1
		movlw		b'11100111'		; Pas de rsistance en pull-up, Int sur up RB0, Clock TMR0=RA4, PSA = Timer0, PSA = 1/256
		movwf		OPTION_REG
		movlw		b'11111000' 	; RA0-RA2 en sorties = N de digit, RA3-RA4 en entres = Longueur du tronon.
		movwf		TRISA
		movlw		b'11110000'		; RB0-RB3 en sorties = Valeur du digit, RB4-RB7 en entres = Capteurs + Interrupteur Tour/Tronon.
		movwf		TRISB
		BANK0

		call		testaffich		; Test  la mise sous tension, allume les 3 leds + 8888.88 pendant 1,3 seconde.

; Elaboration de la variable "choixdistance".
;   Entres RA4-RA3 => Choix tronon de comptage sur 100 cm, 50 cm, 20 cm ou 10 cm ou Comptage sur un tour complet du rseau.
		movlw		1				; Choix du tronon = 100, 50, 20, 10 cm = 4, 3, 2 ou 1. Tour = 0.
		movwf		choixdistance
		btfsc 		RAchdist_1		; RA4 - RA3     : 00=10cm, 01=20cm, 10=50cm, 11=100cm.
		incf		choixdistance,f	; choixdistance =  1=10cm,  2=20cm,  3=50cm,  4=100cm.
		btfsc 		RAchdist_2
		incf		choixdistance,f
		btfsc 		RAchdist_2
		incf		choixdistance,f
		btfss 		RBchtrontr		
		clrf		choixdistance	; 0 = Tour.

; Configuration du Prdidiviseur du Timer1, suivant le type de mesure pour conserver une bonne prcision.
;   Facteur de prdivision : Prescale value= 11=1/8, Prescale value= 10=1/4, Prescale value= 01=1/2, Prescale value= 00=1/1.
		bsf 		T1CON,T1CKPS1
		bsf 		T1CON,T1CKPS0	
		movf    	choixdistance,w
		btfsc   	STATUS,Z
		goto		fin_chprediv	; choixdistance=0 => 1 Tour => Prdiviseur Timer1=> B'11'=1/8
		bcf 		T1CON,T1CKPS1
		movf    	choixdistance,w
		addlw		-3
		btfsc   	STATUS,Z
		goto		fin_chprediv	; choixdistance=3 => 50 cm => Prdiviseur Timer1=> B'01'=1/2
		bcf 		T1CON,T1CKPS0
		bsf 		T1CON,T1CKPS1	
		movf    	choixdistance,w
		addlw		-4
		btfsc   	STATUS,Z
		goto		fin_chprediv	; choixdistance=4 => 100 cm => Prdiviseur Timer1=> B'10'=1/4
		bcf 		T1CON,T1CKPS1
		bcf 		T1CON,T1CKPS0	; choixdistance=1 ou 2 => 10 ou 20 cm => Prdiviseur Timer1=> B'00'=1/1	
fin_chprediv	

; Suite de l'initialisation.
		clrf		digit07			; Vitesse  afficher = 0.00
		clrf		digit1
		clrf		digit2
		clrf		digit3
		clrf		digit4
		clrf		digit5
		clrf		digit6
		clrf		digit7
		clrf		digit8

; Au dmarrage du programme, si un train est devant une barrire, on temporise le temps que le train libre le capteur.
; On fait clignoter la led lentement.
		movf    	choixdistance,w	; Choix du tronon = 100, 50, 20, 10 cm = 4, 3, 2 ou 1. Tour = 0.
		btfsc   	STATUS,Z
		goto		init_5

; 1 / Mesure sur un tronon.
		btfss		RBcapt_1
		goto		init_2			; RBcapt_1=_0 => Train => Tempo
		btfsc		RBcapt_2
		goto		init_9			; RBcapt_2=_0 => Train => Tempo
init_2	bsf 		bit_etat_cll	; bit clignotement lent = 1
		bcf			bit_etat_clr	; bit clignotement rapide = 0
		bsf 		bit_tempotronini ; bit_tempotronini = 1
		clrf		cpttronini
		clrf		cpt100ms	
		bsf 		bit_capt_1		; On fait comme si le train tait pass devant le tronon,
		bsf 		bit_capt_2		; en attente que le convoi complet passe devant les capteurs.
		goto		init_9

; 2 / Mesure sur un tour de rseau.
init_5	btfsc		RBcapt_1
		goto		init_9			; RBcapt_1=_0 => Train => Tempo
		bsf 		bit_etat_cll	; bit clignotement lent = 1
		bcf			bit_etat_clr	; bit clignotement rapide = 0
		bsf 		bit_tempotourini ; bit_tempotourini = 1
		clrf		cpttourini		; cpttourini = 0
		clrf		cpt100ms	
		bcf 		bit_capt_1	
		bsf 		bit_capt_2		; bit_capt_2 = 1 indique que l'on est avant le 1er tour de calcul.
		
; Configuration des interruptions.
init_9	bcf 		INTCON,GIE		; Interruptions globalement interdites
		bcf 		T1CON,TMR1ON	; Arrte le Timer1.
		clrf		PIR1
		clrf		cmpt1
		clrf		cmpt2
		clrf		TMR1H			; Raz des compteurs et du prscaler.
		clrf		TMR1L
		bcf 		PIR1,TMR1IF		; Efface le flag de dbordement du Timer1 = 0.
		BANK1
		clrf		PIE1
		bsf			PIE1,TMR1IE		; Autorise l'interruption sur dbordement du Timer1 = 1.
		BANK0
		clrf		INTCON
		movf		PORTB,w
		movwf		PORTB			; Lecture obligatoire du PortB sur lui mme pour rinitialiser le prochain changement d'tat.
		bcf 		INTCON,RBIF		; Efface le flag de changement d'tat des entres RB4  RB7.
		bsf 		INTCON,RBIE		; Autorise les interruptions sur changement d'tat des entres RB4  RB7 = 1.
		bsf 		INTCON,PEIE		; Interruptions d'entres globalement autorises = 1.
		bsf 		INTCON,GIE		; Toutes les interruptions sont autorises = 1.


;*************************************************************************************
; Boucle principale de rafraichissement de l'affichage, en attente d'interruption.
boucle	call	 	affiche			; Allume les 6 digits + les 3 leds (Dure = 12 msec).
		btfsc		bit_razprg		; bit_razprg=1 ?, Changement de mode demand "Tour/Tronon" ?
		goto 		init			; Oui => Rinit programme.
		
		btfss		bit_calcul		; La routine d'interruption a t'elle demand un calcul de la vitesse ?
		goto		boucle2			; Non !

; Calcul de la vitesse demand !
		bcf 		bit_calcul		; Le calcul de la vitesse se fait qu'une fois, alors raz 'bit_calcul'.
		btfsc		bit_etat_err	; Si les compteurs cmpt2 + cmpt1 ont dbords, alors 'bit_etat_err=1' (Vitesse trop lente),
		goto		calc_err		; et pas de calcul de la vitesse possible.

		movf    	choixdistance,w	; Choix du tronon = 100, 50, 20, 10 cm = 4, 3, 2 ou 1. Tour = 0.
		btfss   	STATUS,Z
		goto		chtrsuite1
		movlw		divi_t6			; choixdistance=0 => Recopie le dividende pour calculer la vitesse sur le tour complet.
		movwf		dividend+0		; MSB
		movlw		divi_t5
		movwf		dividend+1
		movlw		divi_t4
		movwf		dividend+2
		movlw		divi_t3
		movwf		dividend+3
		movlw		divi_t2
		movwf		dividend+4
		movlw		divi_t1
		movwf		dividend+5
		goto		chtrsuite9

chtrsuite1
		movf    	choixdistance,w
		addlw		-4
		btfss   	STATUS,Z
		goto		chtrsuite2
		movlw		divi_m106		; choixdistance=4 => Recopie le dividende pour calculer la vitesse sur le tronon de 100 cm.
		movwf		dividend+0		; MSB
		movlw		divi_m105
		movwf		dividend+1
		movlw		divi_m104
		movwf		dividend+2
		movlw		divi_m103
		movwf		dividend+3
		movlw		divi_m102
		movwf		dividend+4
		movlw		divi_m101
		movwf		dividend+5
		goto		chtrsuite9

chtrsuite2
		movf    	choixdistance,w
		addlw		-3
		btfss   	STATUS,Z
		goto		chtrsuite3
		movlw		divi_m56		; choixdistance=3 => Recopie le dividende pour calculer la vitesse sur le tronon de 50 cm.
		movwf		dividend+0		; MSB
		movlw		divi_m55
		movwf		dividend+1
		movlw		divi_m54
		movwf		dividend+2
		movlw		divi_m53
		movwf		dividend+3
		movlw		divi_m52
		movwf		dividend+4
		movlw		divi_m51
		movwf		dividend+5
		goto		chtrsuite9

chtrsuite3
		movf    	choixdistance,w
		addlw		-2
		btfss   	STATUS,Z
		goto		chtrsuite4
		movlw		divi_m26		; choixdistance=2 => Recopie le dividende pour calculer la vitesse sur le tronon de 20 cm.
		movwf		dividend+0		; MSB
		movlw		divi_m25
		movwf		dividend+1
		movlw		divi_m24
		movwf		dividend+2
		movlw		divi_m23
		movwf		dividend+3
		movlw		divi_m22
		movwf		dividend+4
		movlw		divi_m21
		movwf		dividend+5
		goto		chtrsuite9
		
chtrsuite4
		movf    	choixdistance,w
		addlw		-1
		btfss   	STATUS,Z
		goto		calc_err
		movlw		divi_m16		; choixdistance=1 => Recopie le dividende pour calculer la vitesse sur le tronon de 10 cm.
		movwf		dividend+0		; MSB
		movlw		divi_m15
		movwf		dividend+1
		movlw		divi_m14
		movwf		dividend+2
		movlw		divi_m13
		movwf		dividend+3
		movlw		divi_m12
		movwf		dividend+4
		movlw		divi_m11
		movwf		dividend+5
		
chtrsuite9	
; Les 3 octets "Divisor" sont dj renseigns par la routine d'interruption, il reste plus qu' faire la division.
		call 		Divide_48by24	; Division de 6 octets par 3 octets.  Vitesse = Longueur / Temps

; On vrifie que la vitesse tient sur 3 octets LSB, car la conversion BINtoDEC ne prend que les 3 octets LSB !
;   Si un des 3 octets MSB <> 0 => Erreur de calcul de la vitesse => Affichage impossible.
		movf		dividend+0,w	; MSB Dividend+0=0 ?, Si oui = OK, sinon erreur vitesse trop grande.
		btfss		STATUS,Z
		goto		calc_err
		movf		dividend+1,w	; Dividend+1=0 ?, Si oui = OK, sinon erreur vitesse trop grande.
		btfss		STATUS,Z
		goto		calc_err
		movf		dividend+2,w	; Dividend+2=0 ?, Si oui = OK, sinon erreur vitesse trop grande.
		btfss		STATUS,Z
		goto		calc_err

		movf		dividend+3,w	; MSB sur 3 octet. Recopie le rsultat de la division (Vitesse) pour une conversion en chiffres BCD.
		movwf		count2		
		movf		dividend+4,w
		movwf		count1			
		movf		dividend+5,w	; LSB sur 3 octets.
		movwf		count0				
		call		bin2dec			; Dividend3,4,5 =>  Digit8,7,6,5,4,3,2,1 = Conversion binaire vers dcimal. count0, count1, count2 modifis !!!

		movf		digit8,w		; Digit8=0 ?, Si oui = OK.
		btfss		STATUS,Z
		goto		calc_err		; Non => Erreur vitesse trop grande > 9999.99, Affichage impossible.
		movf		digit7,w		; Digit7=0 ?, Si oui = OK.
		btfss		STATUS,Z
		goto		calc_err		; Non => Erreur vitesse trop grande > 9999.99, Affichage impossible.
		goto		boucle2

calc_err
; En cas d'erreur de mesure, trop d'attente ou calcul impossible, vitesse > 9999.99.
; bit_etat_err=1 + Raz afficheur => Affiche 0.00 + led erreur
; En attente d'un nouveau passage devant les capteurs aprs la tempo de fin de passage du train.
		bsf 		bit_etat_err	; bit_etat_err = 1.
		bcf 		bit_capt_1		; bit_capt_1 = 0
		bcf 		bit_capt_2		; bit_capt_2 = 0
		clrf		digit1		
		clrf		digit2
		clrf		digit3
		clrf		digit4
		clrf		digit5
		clrf		digit6
		movf    	choixdistance,w	; Choix du tronon = 100, 50, 20, 10 cm = 4, 3, 2 ou 1. Tour = 0.
		btfsc   	STATUS,Z
		goto		calc_err2
; 1 / Mesure sur un tronon.
		bsf 		bit_tempotronini ; bit_tempotronini = 1
		clrf		cpttronini		;
		clrf		cpt100ms	
		bsf 		bit_capt_1		; Simulation fin de passage pour temporisation de fin de passage.
		bsf 		bit_capt_2
calc_err2		
		

; On affichera les digits 'digit6'  'digit1' + 'digit07' pour les 3 leds.
; On allume les 3 leds comme on afficherait les segments de l'afficheur n7 :
;   Si tour = 0 et erreur = 0 --> digit07 = 7 : Allume le segment   'B'       = led 'Tronon'
;   Si tour = 0 et erreur = 1 --> digit07 = 2 : Allume les segments 'B' + 'E' = led 'Tronon' + 'Erreur'
;   Si tour = 1 et erreur = 0 --> digit07 = 5 : Allume le segment   'F'       = led 'Tour complet'
;   Si tour = 1 et erreur = 1 --> digit07 = 6 : Allume les segments 'B' + 'F' = led 'Tour complet' + 'Erreur'

;----------------------------------------------------------------------------
; Gestion des temporisations aprs un passage devant les capteurs, pour laisser passer tous les wagons.
boucle2	movf    	choixdistance,w	; Choix du tronon = 100, 50, 20, 10 cm = 4, 3, 2 ou 1. Tour = 0.
		btfsc   	STATUS,Z
		goto		bcl15

; 1 / Mesure sur un tronon.
		btfss		bit_tempotronini
		goto		boucle3			; Rien en cours.
; Attente de fin de passage devant les capteurs sur un tronon.
		incf		cpt100ms,f
		movf		cpt100ms,w
		addlw		-8
		btfss		STATUS,Z
		goto		bcl12
		clrf		cpt100ms		; Raz cpt100ms toutes les 8 * 7 * 1,7 ms = 100 msec.
		incf		cpttronini,f
		
bcl12	btfss		RBcapt_1
		clrf		cpttronini		; RBcapt_1=_0 => Train => Raz tempo.
		btfss		RBcapt_2
		clrf		cpttronini		; RBcapt_2=_0 => Train => Raz tempo.
		
		movf    	cpttronini,w
		addlw		-tempotronini
		btfss   	STATUS,Z
		goto		boucle3
; Fin de tempo,
; Au demarrage du programme, un train tait devant le capteur. Le train a maintenant dgag du capteur. 
; Ou aprs un mesure, x secondes sans passage de train, retour  la normale prt  une nouvelle mesure.
		bcf 		bit_tempotronini ; bit_tempotronini = 0
		bcf 		bit_capt_1		; bit_capt_1 = 0
		bcf 		bit_capt_2		; bit_capt_2 = 0
		bcf 		bit_etat_cll	; bit_etat_cll = 0
		bcf 		bit_etat_clr	; bit_etat_clr = 0
		goto		boucle3

; 2 / Mesure sur un tour de rseau.
bcl15	btfss		bit_tempotourini
		goto		boucle3			; Rien en cours.
		incf		cpt100ms,f
		movf		cpt100ms,w
		addlw		-8
		btfss		STATUS,Z
		goto		bcl16
		clrf		cpt100ms		; Raz cpt100ms toutes les 8 * 7 * 1,7 ms = 100 msec.
		incf		cpttourini,f
		
bcl16	btfss		RBcapt_1
		clrf		cpttourini		; RBcapt_1=_0 => Train => Raz tempo.
		movf    	cpttourini,w
		addlw		-tempotourini
		btfss   	STATUS,Z
		goto		boucle3
; Fin de la tempo.
		btfss		bit_capt_2		
		goto		bcl17
; Au demarrage du programme, un train tait devant le capteur car bit_capt_2=1.
; Le train a maintenant dgag du capteur. On est avant le 1er tour.
		bcf 		bit_capt_1		; bit_capt_1 = 0
		bcf 		bit_capt_2		; bit_capt_2 = 0
		bcf 		bit_tempotourini ; bit_tempotourini = 0
		bcf 		bit_etat_clr	; bit_etat_clr = 0
		bcf 		bit_etat_cll	; bit_etat_cll= 0
		goto		boucle3

bcl17	btfsc		bit_capt_1
		goto		bcl18
; Fin de la tempo du premier passage devant le capteur sur un tour de rseau.
; x secondes sans passage de train.
		bsf 		bit_capt_1		; bit_capt_1 = 1
		bcf 		bit_tempotourini ; bit_tempotourini = 0
		goto		boucle3 		; On laisse la led clignoter lentement ou rapidement.

; Attente de fin des passages suivants devant le capteur sur un tour de rseau.
; Fin des passages suivants devant le capteur sur un tour de rseau.
; x secondes sans passage de train.
bcl18	bcf 		bit_tempotourini ; bit_tempotourini = 0
		
boucle3	goto		boucle			; On laisse la led clignoter lentement.
		
		
;*************************************************************************************
;*************************************************************************************
; Routine d'interruption.
;   Interruption sur RB(47) ou dbordement du Timer1 (16 bits) ?
; 
interrup
		bcf 		INTCON,GIE		; Inhibe toutes les interruptions.
		movwf   	w_temp  		; Sauver registre w.
		swapf		STATUS,w		; Swap status avec rsultat dans w.
		movwf		status_temp		; Sauver status swapp.

		btfss		INTCON,RBIF		; Interrupt RB47 en cours ?
		goto 		int2			; Non !
		call		int_rb47		; Oui => Traiter interrupt RB47.
		movf		PORTB,w
		movwf		PORTB			; Lecture obligatoire du PortB sur lui mme pour rinitialiser le prochain changement d'tat.
		bcf 		INTCON,RBIF		; Effacer flag interrupt RB47.

int2	btfss		PIR1,TMR1IF		; Interrupt Timer1 en cours ?
		goto		restreg			; Non !
		call		int_timer1		; Oui => Traiter interrupt Timer1.
		bcf 		PIR1,TMR1IF		; Effacer flag interupt Timer1.

restreg	swapf		status_temp,w	; Swap ancien status, rsultat dans w.
		movwf		STATUS			; Restaurer status.
		swapf		w_temp,f		; Inversion L et H de l'ancien W sans modifier Z.
		swapf		w_temp,w  		; Rinversion de L et H dans W, W restaur sans modifier status.
		retfie  					; Fin routine d'interruption.

;----------------------------------------------------------------------------
; Routine d'interruption.
;   Une entre RB(4-7) a chang d'tat, laquelle ?
int_rb47
		btfsc 		RBchtrontr		; La position de l'interrupteur "Tour/Tronon" a t'elle change ?
		goto		intrb4a
		movf		choixdistance,w	
		btfss 		STATUS,Z
		bsf 		bit_razprg		; Inter=1(Tronon)+choixdistance=0(Tour)  => bit_razprg=1.
		goto		intrb4b
intrb4a	movf		choixdistance,w
		btfsc 		STATUS,Z
		bsf 		bit_razprg		; Inter=0(Tour)+choixdistance<>0(Tronon) => bit_razprg=1.

intrb4b	movf		choixdistance,w	
		btfsc 		STATUS,Z
		goto		intrb5a			; choixdistance=0 => Mesure sur tour du rseau.
; 1 / Mesure sur tronon.	
		btfss		bit_capt_1
		goto		intrb4d
		btfsc		bit_capt_2
		return						; bit_capt_1=1 et bit_capt_2=1 => Passage devant le tronon dja effectu, rien  faire. 
									; En attente de fin de la tempo dans la boucle principale pour le retour  la normale.
intrb4d	btfsc		bit_capt_1
		goto		intrb4f
		btfsc		bit_capt_2
		goto		intrb4f		
		btfsc		RBcapt_1
		goto		intrb4e			; Pas de 1er passage devant capt_1.
; bit_capt1=0 et bit_capt2=0 et RBcapt_1=_0.
; Mesure sur tronon, premier passage devant capt1.
;   Raz et dbut de comptage Timer1 + cmpt1 + cmpt2.
		clrf		cmpt1
		clrf		cmpt2
		clrf		TMR1H			; Raz des compteurs et du prscaler.
		clrf		TMR1L
		bsf 		T1CON,TMR1ON	; Met en marche le Timer1.
		bcf 		PIR1,TMR1IF		; Effacer flag interupt Timer1.
		bsf			bit_etat_clr	; bit clignotement rapide = 1
		bcf 		bit_etat_cll	; bit clignotement lent = 0
		bcf 		bit_etat_err	; bit_led_erreur = 0
		bsf 		bit_capt_1		; 1er passage devant capteur1 effectu.
		return

intrb4e	btfsc		RBcapt_2
		return
; bit_capt1=0 et bit_capt2=0 et RBcapt_2=_0.
; Mesure sur tronon, premier passage devant capt2.
;   Raz et dbut de comptage Timer1 + cmpt1 + cmpt2.
		clrf		cmpt1
		clrf		cmpt2
		clrf		TMR1H			; raz des compteurs et du prscaler
		clrf		TMR1L
		bsf 		T1CON,TMR1ON	; mettre en marche le timer1 (TMR1L et TMR1H dj gaux  0) 
		bcf 		PIR1,TMR1IF		; Effacer flag interupt Timer1.
		bsf			bit_etat_clr	; bit clignotement rapide = 1
		bcf 		bit_etat_cll	; bit clignotement lent = 0
		bcf 		bit_etat_err	; bit_led_erreur = 0
		bsf 		bit_capt_2		; Passage devant capteur2 effectu.
		return

intrb4f	btfss		bit_capt_1
		goto		intrb4h
		btfss		RBcapt_2
		goto		intrb4j			; bit_capt_1=1 et RBcapt_2=_0
		return
		
intrb4h	btfss		bit_capt_2
		return
		btfsc		RBcapt_1
		return
; (bit_capt_1=1 et RBcapt_2=_0) ou (bit_capt_2=1 et RBcapt_2=_0).		
; Mesure sur tronon, bit_capt_1=1 et bit_capt_2=1;
;   Le passage devant le tronon vient juste d'tre effectu.
;   Au maximum cmpt2 = '00' + cmpt1 = 'FF' + TMR1 = 'FFFF'. 
intrb4j	bcf 		T1CON,TMR1ON	; Arrte le timer1 et effacer flag interrupt timer1 plus loin.
		movf		TMR1L,w			; Sauvegarde des trois compteurs
		movwf		divisor+2		; LSB
		movf		TMR1H,w			
		movwf		divisor+1
		movf		cmpt1,w
		movwf		divisor+0		; MSB
		btfsc		PIR1,TMR1IF		; Si le compteur TMR1 vient de dborder alors que l'on
		incf		divisor+0,f		; n'est pas pass dans la routine d'incrmentation des compteurs
		bcf 		PIR1,TMR1IF		; Effacer flag interrupt timer1
		bsf 		bit_calcul		; Calcul de la vitesse demande.
		bsf 		bit_etat_cll	; bit clignotement lent = 1 en attente du passage complet du train.
		bcf			bit_etat_clr	; bit clignotement rapide = 0
		bsf 		bit_tempotronini ; bit_tempotronini = 1
		clrf		cpttronini		;
		clrf		cpt100ms	
		bsf 		bit_capt_1		; Passage devant capteur1 ou capteur2 effectu.
		bsf 		bit_capt_2
		return

; 2 / Mesure sur un tour de rseau.	
intrb5a	btfsc		RBcapt_1
		return		

; RBcapt_1=_0.	
intrb5d	btfsc		bit_tempotourini
		return
		btfsc		bit_capt_1
		goto		intrb5e
; Mesure sur un tour de rseau, premier passage devant capt1.
;   Raz et dbut de comptage Timer1 + cmpt1 + cmpt2.
		clrf		cmpt1
		clrf		cmpt2
		clrf		TMR1H			; Raz des compteurs et du prscaler
		clrf		TMR1L
		bsf 		T1CON,TMR1ON	; Mettre en marche le timer1 (TMR1L et TMR1H dj gaux  0) 
		bcf 		PIR1,TMR1IF		; Effacer flag interrupt timer1
		bsf			bit_etat_clr	; bit clignotement rapide = 1 Clignotement rapide au 1er passage
		bcf 		bit_etat_cll	; bit clignotement lent = 0
		bsf 		bit_tempotourini ; bit_tempotourini = 1
		clrf		cpttourini		; cpttourini = 0
		bcf 		bit_etat_err	; bit_led_erreur = 0
		return					

; Mesure sur un tour de rseau, fin des passages suivants devant capt1.
;   Le passage devant le capteur vient juste d'tre effectu.
;   Au maximum cmpt2 = '03' + cmpt1 = 'FF' + TMR1 = 'FFFF'. 
intrb5e	bcf 		T1CON,TMR1ON	; Arrte le timer1 et effacer flag interrupt timer1 plus loin.
		movf		TMR1L,w			; Sauvegarde des trois compteurs
		movwf		divisor+2		; LSB
		movf		TMR1H,w			
		movwf		divisor+1
		movf		cmpt1,w
		movwf		divisor+0		; MSB
		btfsc		PIR1,TMR1IF		; Si le compteur TMR1 vient de dborder alors que l'on
		incf		divisor+0,f		; n'est pas pass dans la routine d'incrmentation des compteurs.

; Au maximum cmpt2 = '03' + cmpt1 = 'FF' + TMR1 = 'FFFF'. 
; On dcale le tout de 2 bits  droite, pour faire tenir dans 3 octets.
		bcf 		STATUS,C		; Clear CARRY. 2 dcalages de 2 bits vers la droite = 2 divisions par 2.
		rrf 		cmpt2,f			; MSB
		rrf 		divisor+0,f
		rrf 		divisor+1,f
		rrf 		divisor+2,f		; LSB
		
		bcf 		STATUS,C		; Clear CARRY.
		rrf 		cmpt2,f			; MSB
		rrf 		divisor+0,f
		rrf 		divisor+1,f
		rrf 		divisor+2,f		; LSB

		bsf 		bit_calcul		; Calcul de la vitesse demande.
		bsf 		bit_etat_cll	; bit clignotement lent = 1 Clignotement lent aux passages suivants.
		bcf			bit_etat_clr	; bit clignotement rapide = 0
		bsf 		bit_tempotourini ; bit_tempotourini = 1
		clrf		cpttourini		; cpttourini = 0
		clrf		cpt100ms	
	
;   Raz et dbut de comptage Timer1 + cmpt1 + cmpt2.
		clrf		cmpt1
		clrf		cmpt2
		clrf		TMR1H			; Raz des compteurs et du prscaler
		clrf		TMR1L
		bsf 		T1CON,TMR1ON	; Mettre en marche le timer1 (TMR1L et TMR1H dj gaux  0) 
		bcf 		PIR1,TMR1IF		; Effacer flag interrupt timer1
		return

;----------------------------------------------------------------------------
; Routine d'interruption.
;   Le Timer1 (16 bits) a dbord, alors on incrmente les compteurs cmpt1 + cmpt2.
int_timer1	
		incf		cmpt2,f			; Incrmente : cmpt2 + cmpt1.
		incfsz		cmpt1,f
		decf		cmpt2,f

		movf		choixdistance,w	; 0 = Tour.
		btfsc 		STATUS,Z		; Position de l'interrupteur "Tour/Tronon" ?
		goto		i_time5			; => Mesure sur tour du rseau.

; 1 / Mesure sur tronon.	
		movf		cmpt2,w
		btfsc 		STATUS,Z	
		return
; cmpt2<>0 => Dbordement cmpt1 => Blocage compteur Timer1.
; Maximum cmpt2 + cmpt1 + timer1 = 0x00 0xFF, 0xFFFF = 17  67 secondes pour un vitesse minimum de 1,9  4,7 km/h.
i_time0	bcf 		T1CON,TMR1ON	; Arrte le timer1
		bcf 		bit_capt_1		; bit_capt_1 = 0
		bcf 		bit_capt_2		; bit_capt_2 = 0
		bcf 		bit_etat_cll	; bit clignotement lent = 0
		bcf			bit_etat_clr	; bit clignotement rapide = 0
		bsf 		bit_etat_err	; bit_led_erreur = 1
		bsf 		bit_calcul		; Calcul de la vitesse demande.
		bcf 		bit_tempotronini ; bit_tempotronini = 0
		clrf		cpttronini		; cpttronini = 0
		clrf		cpt100ms			
		return

; 2 / Mesure sur un tour de rseau. 
i_time5	movf    	cmpt2,w
		addlw		-4
		btfss   	STATUS,Z	 	; cmpt1=4 => Trop d'attente => Blocage compteur Timer1.
		return
; cmpt2 => trop d'attente => Blocage compteur Timer1.
; Maximum cmpt2 + cmpt1 + timer1 = 0x03 0xFF, 0xFFFF = 10 mn.
		bcf 		T1CON,TMR1ON	; Arrte le timer1
		bcf 		bit_capt_1		; bit_capt_1 = 0
		bcf 		bit_capt_2		; bit_capt_2 = 0
		bcf 		bit_etat_cll	; bit clignotement lent = 0
		bcf			bit_etat_clr	; bit clignotement rapide = 0
		bsf 		bit_etat_err	; bit_led_erreur = 1
		bsf 		bit_calcul		; Calcul de la vitesse demande.
		bcf 		bit_tempotourini ; bit_tempotourini = 0
		clrf		cpttourini		; cpttourini = 0
		clrf		cpt100ms	
		return

		
;*************************************************************************************
;*************************************************************************************
; SOUS-ROUTINE DE TEST DE l'affichage au lancement du programme
;   Routine d'affichage de 8888.88 pendant 1 seconde.
;   Modifie digit1  digit8 et digit07.

testaffich	
		movlw		.8
		movwf		digit1
		movwf		digit2
		movwf		digit3
		movwf		digit4
		movwf		digit5
		movwf		digit6
		movwf		digit07			; 8 => les 3 leds rouge, orange et verte.
		movlw		.100			; 100 tours de 14 ms => 1,4 secondes  4Mhz.
		movwf		digit8
testaf2 call 		affiche6		; Va en "affiche6" pour ne pas modifier digit07 avec les valeurs relles des leds.
		decfsz  	digit8,f		; Dcrmenter compteur2
		goto		testaf2		; Si <> 0, boucler
		return


;*************************************************************************************
;*************************************************************************************
; SOUS-ROUTINE D'AFFICHAGE
; Routine d'affichage sur le port multiplx. Dure 12 msec.
; 2 msec par digit = 1,7*7 ms pour tout afficher = 12 ms = 80 Hz de frquence de rafraichissement pour viter le scintillement.
; Utilise la variable "bit_digitzero" pour ne pas afficher les zros non significatifs des units.  Affiche 0.00 au minimum.
;
; Entres :
;   digit6 + digit5 + digit4 + digit3 + digit2 + digit1 = 0  9, digit1 = LSB. 
;   digit07 = 3 leds.
; Le cblage du digit07 sert  allumer les trois leds. Les segments B, E et F correspondent aux leds rouge, orange et vert.
; Sur ce digit07, on affiche les chiffres 7, 5, 2 ou 6 en fonction des leds  allumer.
;
; Variables modifies :
;   bit_digitzero = 0/1 = Pour ne pas afficher les zros non significatifs.
;   clignot
; Calcul de l'affichages des les d'tat.
;   Allume la led "Tour" ou la led "Tronon" suivant choixdistance.
;   Allume la led "Erreur" si "bit_etat_err=1".
;   Fixe ou clignote rapide si "bit_etat_clr=1" ou clignote lentement si "bit_etat_cll=1".
;   Tronon = Vert => Afficher '7'.
;   Tronon = Vert + Erreur Rouge => Afficher '2'
;   Tour = Orange => Afficher '5'
;   Tour = Orange + Erreur Rouge =>  Afficher '6'

affiche
		movf    	choixdistance,w	; Choix du tronon = 100, 50, 20, 10 cm = 4, 3, 2 ou 1. Tour = 0.
		btfsc   	STATUS,Z
		goto		aff_t1
		movlw		7				; Tronon = Vert => Afficher '7'.
		btfsc		bit_etat_err
		movlw		2				; Tronon = Vert + Erreur Rouge => Afficher '2'
		goto		aff_t5
aff_t1	movlw		5				; Tour = Orange => Afficher '5'
		btfsc		bit_etat_err
		movlw		6				; Tour = Orange + Erreur Rouge =>  Afficher '6'
aff_t5	movwf		digit07

; Programme d'affichage des 6 digits + 3 leds.		
affiche6							; On peut entrer dans le programme ici, en forant l'tat du digit07 des leds.
		movlw		7				; 7 => Pour teindre l'afficheur.
		movwf		PORTA
		bcf 		bit_digitzero	; bit_digitzero = 0!
		movf		digit6,w
		btfsc		STATUS,Z		; digit6 == 0 ? 
		goto		affiche5		; oui --> on passe au digit suivant, l'afficheur restera tient pendant 2 msec.
		bsf 		bit_digitzero	; non --> bit_digitzero = 1 ! => On doit afficher ce digit et les prochains digits.
		movf		digit6,w		; Il ne faut pas crire dans RB4  RB7, sinon a fausse l'interruption RB47 !
		call		affiPB
		movlw		5
		movwf		PORTA			; Millier = Affiche le digit n6.
affiche5
		call		tempo2ms		; Laisse le digit Xxxx.xx allum pendant 2 msec.

		btfsc		bit_digitzero	; bit_digitzero = 1 ? => On doit afficher ce digit et les prochains digits.
		goto		affiche5b
		movf		digit5,w
		btfsc		STATUS,Z		; digit5 == 0 ? 
		goto		affiche4		; oui --> on passe au digit suivant
		bsf 		bit_digitzero	; non --> bit_digitzero = 1 ! => On doit afficher ce digit et les prochains digits.
affiche5b
		movlw		7
		movwf		PORTA			; 7 => Pour teindre l'afficheur. Rien branch en sortie n7.	
		movf		digit5,w
		call		affiPB
		movlw		4
		movwf		PORTA			; Centaine = Affiche le digit n5.	
affiche4
		call		tempo2ms		; Laisse le digit xXxx.xx allum pendant 2 msec.

		btfsc		bit_digitzero	; bit_digitzero,0 = 1 --> affiche le digit
		goto		affiche4b
		movf		digit4,w
		btfsc		STATUS,Z		; digit4 == 0 ? 
		goto		affiche3		; oui --> on passe au digit suivant
		bsf 		bit_digitzero	; non --> bit_digitzero = 1 ! => On doit afficher ce digit et les prochains digits.
affiche4b
		movlw		7
		movwf		PORTA			; 7 => Pour teindre l'afficheur.
		movf		digit4,w
		call		affiPB
		movlw		3
		movwf		PORTA			; Dizaine = Affiche le digit n4.		
affiche3

		call		tempo2ms		; Laisse le digit xxXx.xx allum pendant 2 msec.
		movlw		7
		movwf		PORTA			; 7 => Pour teindre l'afficheur.
		movf		digit3,w
		call		affiPB
		movlw		2
		movwf		PORTA			; Unit = Affiche le digit n3.
		call		tempo2ms		; Laisse le digit xxxX.xx allum pendant 2 msec.

		movlw		7
		movwf		PORTA			; 7 => Pour teindre l'afficheur.	
		movf		digit2,w
		call		affiPB
		movlw		1
		movwf		PORTA			; Dixime = Affiche le digit n2.	
		call		tempo2ms		; Laisse le digit xxxx.Xx allum pendant 2 msec.

		movlw		7
		movwf		PORTA			; 7 => Pour teindre l'afficheur.	
		movf		digit1,w
		call		affiPB
		movlw		0
		movwf		PORTA			; Centime = Affiche le digit n1.
		call		tempo2ms		; Laisse le digit xxxx.xX allum pendant 2 msec.

		movlw		7
		movwf		PORTA			; 7 => Pour teindre l'afficheur.
		movf		digit07,w
		call		affiPB
		incf		clignot,f
		btfss		bit_etat_clr	;
		goto		affiche07b
		btfsc		clignot,3		; Pour clignoter rapidement => T = 16* 2ms =192ms => F= 5Hz.
		goto		affiche07e
		goto		affiche07c
affiche07b
		btfss		bit_etat_cll	;
		goto		affiche07c
		btfsc		clignot,5		; Pour clignoter lentement => T = 64*12ms =768ms => F= 1,3Hz.
		goto		affiche07e
affiche07c
		movlw		6   			; digit07 = Leds 1 Tour + Led Erreur + Led 1 mtre.
		movwf		PORTA	
affiche07e
		call		tempo2ms		; Laisse les leds allumes pendant 2 msec.
		return

; Sous routine de modification partielle du PortB, sans toucher aux autres bits.

affiPB	movwf		digitmp
		btfsc		digitmp,0
		bsf			PORTB,0
		btfss		digitmp,0
		bcf			PORTB,0
		btfsc		digitmp,1
		bsf			PORTB,1
		btfss		digitmp,1
		bcf			PORTB,1
		btfsc		digitmp,2
		bsf			PORTB,2
		btfss		digitmp,2
		bcf			PORTB,2
		btfsc		digitmp,3
		bsf			PORTB,3
		btfss		digitmp,3
		bcf			PORTB,3
		return		
		
		
;*************************************************************************************
;*************************************************************************************
; SOUS-ROUTINES DE TEMPORISATION
; Cette temporisation n'a pas besoin d'tre prcise pour assurer le multiplexage des afficheurs.
; Elles ne reoivent aucun paramtre et n'en retourne aucun.
; Ces sous-routine introduisent un retard approximatif de
;   tempo256u  =  256   micro secondes (0,000256 sec.)
;   tempo2ms   =    1,7 milli secondes (0,0017 sec.)
; Modifie les variables "delai1" et "delai2".
 
tempo256u
		movlw		.60  		; 256 s = environ 60 boucles  Horloge = 4Mhz
		movwf		delai1		; effacer compteur1
tempo_a	nop  			  		; perdre 1 cycle
		decfsz  	delai1,f	; dcrmenter compteur1
		goto		tempo_a		; si pas 0, boucler
		return

tempo2ms
		movlw		.7  		; 1,7 ms => Environ 80 Hz de frquence de rafraichissement pour 7 afficheurs.
		movwf		delai2
tempo_b	call		tempo256u
		decfsz  	delai2,f	; Dcrmenter delai2.
		goto		tempo_b		; Si pas 0, boucler
		return

;*************************************************************************************
;*************************************************************************************
; SOUS-ROUTINE DE CONVERSION BINAIRE VERS DECIMAL
; From Peter Hemsley 15July01     http://www.piclist.com/techref/cpus.htm
; Binary to decimal
; Convert 24 bit binary (count0,1,2) to decimal digits 1 to 8
; Revised version. My thanks to all who contributed to the bin to dec discussion in EPE.
;   count0 (lsb)  count2 (msb)  !!! ATTENTION les variables Count0  Count2 sont modifis !!!
;   digit0 (lsd)  digit8 (msd)
;   bitcnt et digcnt

bin2dec	call		clrdig
		movlw		.24		 	;24 bits to do
		movwf		bitcnt
bitlp  	rlf 		count0,f	;shift msb into carry
		rlf 		count1,f
		rlf 		count2,f
		movlw		digit1
		movwf		FSR			;Pointer to digits
		movlw		.8			;8 digits to do
		movwf		digcnt
adjlp  	rlf 		INDF,f		;shift digit 1 bit left
		movlw		-.10
		addwf		INDF,w		;Check and adjust for decimal overflow
		skpnc
		movwf		INDF
		incf		FSR,f		;Next digit
		decfsz  	digcnt,f
		goto		adjlp
		decfsz  	bitcnt,f 	;Next bit
		goto		bitlp
		return
clrdig	clrf		digit1
		clrf		digit2
		clrf		digit3
		clrf		digit4
		clrf		digit5
		clrf		digit6
		clrf		digit7
		clrf		digit8
		return

;*************************************************************************************
;*************************************************************************************
; Divide_48by24 - Routine to divide a 48 bit number with a 24 bit number
; http://www.piclist.com/techref/microchip/math/div/48by24ng.htm
; result upto 48 bits !
; Formula:      Dividend = Dividend/Divisor
; Format:       Little endian, Ram = msb, Ram+x = lsb ( where x=bytesize-1)
; Ram used:     Dividend - 6 bytes ( 48 bits )
;               Divisor  - 3 bytes ( 24 bits )
;               temp     - 3 bytes 
;               temp2    - 1 bytes 
;               bitcount - 1 byte  for loop counting ( and temp. saving of carry )
; Divisor is preserved,  Dividend holds result
; Returns with zero in W if failed ( division with zero )
; Else returns with one in w
; Based on pseudo-code from Nikolai Golovchenko [golovchenko@mail.ru]
 
Divide_48by24
		MOVF		divisor,W		; Test for zero division
		IORWF		divisor+1,W
		IORWF		divisor+2,W
		BTFSC		STATUS,Z
		RETLW		0x00			; divisor = zero, not possible to calculate return with zero in w
		CLRF		temp			; prepare used variables
		CLRF		temp+1
		CLRF		temp+2
		clrf		temp2
		MOVLW		.48				; initialize bit counter (48 dcimal)
		MOVWF		bitcount
Divide_LOOP_48by24
		RLF 		dividend+5,F
		RLF 		dividend+4,F
		RLF 		dividend+3,F
		RLF 		dividend+2,F
		RLF 		dividend+1,F
		RLF 		dividend,F
		RLF 		temp+2,F		; shift in highest bit from dividend through carry in temp
		RLF 		temp+1,F
		RLF 		temp,F
		rlf 		temp2, f
		MOVF		divisor+2,W		; get LSB of divisor
		btfsc		temp2, 7
		goto		Div48by24_add	; subtract 24 bit divisor from 24 bit temp
		SUBWF		temp+2,F		; subtract	
		MOVF		divisor+1,W		; get middle byte
		SKPC						;	if overflow ( from prev.subtraction )
		INCFSZ  	divisor+1,W		; incresase source
		SUBWF		temp+1,F		; and subtract from dest.
		MOVF		divisor,W		; get top byte
		SKPC						;	if overflow ( from prev.subtraction )
		INCFSZ  	divisor,W		; increase source	
		SUBWF		temp,F  		; and subtract from dest.
		movlw   	.1
		SKPC
		subwf		temp2, f
		GOTO		Divide_SKIP_48by24 ; carry was set, subtraction ok, continue with next bit
Div48by24_add						; result of subtraction was negative restore temp
		ADDWF		temp+2,F		; add it to the lsb of temp
		MOVF		divisor+1,W		; middle byte
		BTFSC		STATUS,C		; check carry for overflow from previous addition
		INCFSZ  	divisor+1,W		; if carry set we add 1 to the source
		ADDWF		temp+1,F		; and add it if not zero in that case Product+Multipland=Product
		MOVF		divisor,W		; MSB byte
		BTFSC		STATUS,C		; check carry for overflow from previous addition
		INCFSZ  	divisor,W
		ADDWF		temp,F  		; handle overflow	
		movlw   	.1
		skpnc
		addwf		temp2, f
Divide_SKIP_48by24
		DECFSZ  	bitcount,F		; decrement loop counter
		GOTO		Divide_LOOP_48by24	; another run
		RLF 		dividend+5,F	; finally shift in the last carry
		RLF 		dividend+4,F
		RLF 		dividend+3,F
		RLF 		dividend+2,F
		RLF 		dividend+1,F
		RLF 		dividend,F
		RETLW   	0x01			; done	
		end